/* 
 *           Examplefile for usage of SAPI-Library 
 *
 *      The code is not giving a compact reader, but the target
 *      is to show the usage of many SAPI-functions . 
 *      So, for example, the 'file rewind' doesn't make any sense
 *      in this context .
 *      Also for the reason of training it's not (!!) optimal coding.
 *      Code can be speeded up by local storing of function-results !!
 *
 *      The executable may be startet through commandlineoptions
 *      or by using one of the attached scripts (*.scr). 
 *
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "sapi.h"

/* ERRORPRINT != 0 will force erroroutputs on stderr, if something is going wrong
                   during filehandling                                           */
                   
#define ERRORPRINT 1




/* Some mathematical macros */ 

#ifndef PI
#define PI 3.14159265359
#endif
#define RAD_TO_DEG(a) ((a>=+0.0)?                                \
                                (double)((180.0*a)/PI)           \
                               :                                 \
                                (double)(360.0 - ((-180.0*a)/PI)) )

#define RAD_TO_PLUMINUS_DEG(a) ((double)((180.0*a)/PI))


/* A function, to achieve absolute Beampositions */

static void calculateAbsoluteBeamPosition(double posX,double posY,double heading,
                                          double posAhead,double posAstar,
                                          long posPresentation,
                                          double* resultX,double* resultY);


/* Functions for presenting timedata */

static char* printStartTime(double absTime);
static char* printSoundingTime(double absTime);


char surfDataPath[256] = {" "}; 
char filename[256] = {" "}; 


int main (int argc,char ** argv)
{
 SurfGlobalData*        toGlobalData;
 SurfPositionAnySensor* toPositionSensor;
 SurfEventValues*       toEvent;
 SurfSoundingData*      toSounding;
 SurfCenterPosition*    toCenterPosition;
 SurfMultiBeamDepth*    toMBDepth;

 SurfTransducerParameterTable* toTransducer;
 SurfMultiBeamAngleTable*      toAngles;
 SurfCProfileTable*            toCProfile;
 
 char* sp;
 int n,nrSounding;
 u_short soundingFlag,depthFlag;
 long status,ii,nrBeams;
 double depth,refPosX,refPosY,cPosX,cPosY,posX,posY,posAhead,posAstar;
 double heave,roll,pitch,heading;


 /* Get Path and Filename (without extention '.six or .sda') from shell */

 
 for(n=1;n<argc;n++)
 {
  if((sp = strstr(argv[n],"-F")) != NULL) strncpy(filename,sp+2,255);
  if((sp = strstr(argv[n],"-D")) != NULL) strncpy(surfDataPath,sp+2,255);
 }


 /* Maybe you like to printout actual Library-Versions */

 SAPI_printAPIandSURFversion();
  


 /* SAPI_open will open the files and, if successful, give access to six-Data
    and to the first sounding                                                */
    

 if(SAPI_open(surfDataPath,filename,ERRORPRINT) == 0)
 {

  /* Accessing some six-Data */


  /* few basic parameters */
  
  fprintf(stdout,"SURF Name Of Ship   : '%s'\n",SAPI_getNameOfShip());
  fprintf(stdout,"SURF Type Of Sounder: '%s'\n",SAPI_getTypeOfSounder());
  fprintf(stdout,"SURF Name Of Sounder: '%s'\n",SAPI_getNameOfSounder());
  fprintf(stdout,"SURF Nr Of Soundings: %d\n",SAPI_getNrSoundings());
  fprintf(stdout,"SURF Nr Of Beams    : %d\n",SAPI_getNrBeams());


  /* How to get more global Informations */

  toGlobalData = SAPI_getGlobalData();
  if(toGlobalData != NULL)
  {
   fprintf(stdout,"SURF-GLOBAL_DATA:\n");
   fprintf(stdout,"    Start Time Of Profile  '%s'\n",
                                          printStartTime(SAPI_getAbsoluteStartTimeOfProfile()));
   fprintf(stdout,"    Ellipsoid  '%s'\n",toGlobalData->nameOfEllipsoid);
   fprintf(stdout,"    Projection '%s'\n",toGlobalData->projection);

   refPosX=toGlobalData->referenceOfPositionX;
   refPosY=toGlobalData->referenceOfPositionY;
   if(SAPI_posPresentationIsRad() == 0)
   {                                       /* Presentation in [m] */
    fprintf(stdout,"    PosRef X: %10.0f m  Y: %10.0f m\n",
                  refPosX,
                  refPosY);
   }
   else               
   {                                       /* Presentation in [rad] */
    fprintf(stdout,"    PosRef East: %10.7f deg  North: %10.7f deg\n",
                  RAD_TO_PLUMINUS_DEG(refPosX),
                  RAD_TO_PLUMINUS_DEG(refPosY) );
   }
  }

  
  /* A quick look, how many and which Positionsensors have been stored */
  
  fprintf(stdout,"SURF Nr Of Positionsensors      : %d\n",SAPI_getNrPositionsensors());
  for(ii=0;ii<SAPI_getNrPositionsensors();ii++)
  {
   if((toPositionSensor=SAPI_getPositionSensor(ii)) != NULL)
       fprintf(stdout,"    Sensor Nr.%d = '%s'\n",ii,toPositionSensor->positionSensorName);
  }


  /* A quick look, if SoundvelocityProfiles are available */
  
  fprintf(stdout,"SURF Nr Of SoundvelocityProfiles: %d\n",SAPI_getNrSoundvelocityProfiles());


  /* Are there any Eventrecordings ?? */
  
  fprintf(stdout,"SURF Nr Of Events               : %d\n",SAPI_getNrEvents());
  for(ii=0;ii<SAPI_getNrEvents();ii++)
  {
   if((toEvent=SAPI_getEvent(ii)) != NULL)
       fprintf(stdout,"    Event Nr.%d time %10.2f sec '%s'\n",ii,toEvent->relTime,toEvent->text);
  }


  /* These Polygons, if calculated, are representing the covered Area of this Profile */
  
  fprintf(stdout,"SURF Nr Of Polygonelements      : %d\n",SAPI_getNrPolygonElements());

  fprintf(stdout,"\n\n");




  /* Now lets go to read some sda-Data (Sounding-Data) */
 
  nrSounding = 0;
  nrBeams = SAPI_getNrBeams();
  status = 0;
  while(status == 0)    /* This loop is stepping sounding by sounding */
  {

   /* Get some Basic Sounding-Data */

   toSounding = SAPI_getSoundingData();
   fprintf(stdout,"Sounding Nr %d   Profiletime: '%s'\n",nrSounding,
                    printSoundingTime(SAPI_getAbsoluteStartTimeOfProfile() + toSounding->relTime));
   soundingFlag = toSounding->soundingFlag;


   /* How the Soundingflag works */
    
   if((soundingFlag & SF_DELETED) != 0)
   {
    fprintf(stdout,"  Sounding deleted\n");
   }
   else
   {
    heading = (double)toSounding->headingWhileTransmitting;
    heave = (double)toSounding->heaveWhileTransmitting;
    roll  = (double)toSounding->rollWhileTransmitting;
    pitch = (double)toSounding->pitchWhileTransmitting;
    fprintf(stdout,"  Heading: %5.2f deg Heave: %5.2f m Roll: %5.2f deg Pitch: %5.2f deg\n",
                  RAD_TO_DEG(heading),
                  heave,
                  RAD_TO_PLUMINUS_DEG(roll),
                  RAD_TO_PLUMINUS_DEG(pitch) );


    /* Different Types of Soundingdata can be stored */
    
    if(strcmp(SAPI_getTypeOfSounder(),"F") == 0) /* is it a FAN-Type Sounder ? */
    {

     /* What kind of a fan do we have ? */
     
     if((soundingFlag & SF_FAN_PAT_1) == 0)
     {
      fprintf(stdout,"  Fullfan\n");
     }
     else
     {
      if((soundingFlag & SF_FAN_PAT_2) == 0)
      {
       fprintf(stdout,"  Splitfan Port\n");
      }
      else
      {
       fprintf(stdout,"  Splitfan Starboard\n");
      }
     }


     /* Accessing the center-position (Positionsensor 0 is the selected Surveysensor) */ 

     if((toCenterPosition=SAPI_getCenterPosition((long)0)) == 0)
     {
      fprintf(stderr,"ExampleERROR: Positionsensor for this Sensornumber not available!\n");
     }
     else
     {


      /* Positiondata in CenterPositions are relativ to the positionreference ! */
      
      cPosX=(double)toCenterPosition->centerPositionX + refPosX;     
      cPosY=(double)toCenterPosition->centerPositionY + refPosY; 

      if(SAPI_posPresentationIsRad() == 0)
      {                                       /* Presentation in [m] */
       fprintf(stdout,"  CenterPos X: %10.0f m  Y: %10.0f m\n",
                  cPosX,
                  cPosY);
      }
      else               
      {                                       /* Presentation in [rad] */

       /* if you are going through the DATELINE you should check for +/-PI ! */

       fprintf(stdout,"  CenterPos East: %10.7f deg  North: %10.7f deg\n",
                  RAD_TO_PLUMINUS_DEG(cPosX),
                  RAD_TO_PLUMINUS_DEG(cPosY) );
      }

      /* Now lets look for Beamdata */
     
      if(nrSounding < 5)                 /* only some soundings to reduce DATA-garbage */
      {
       for(ii=0;ii<nrBeams;ii++)
       {
       
        /* Get Access to the Beam-Data */
        
        if((toMBDepth=SAPI_getMultiBeamDepth((long)ii)) != NULL)
        {
         depthFlag = (u_short)toMBDepth->depthFlag;

         /* How to handle the depthFlag */

         if((depthFlag & (SB_DELETED+SB_DEPTH_SUPPRESSED+SB_REDUCED_FAN)) == 0)
         {
          depth     = (double) toMBDepth->depth;
          posAhead  = (double) toMBDepth->beamPositionAhead;
          posAstar  = (double) toMBDepth->beamPositionStar;

          /* Transforming shiprel. Positions into absolute Positions */

          calculateAbsoluteBeamPosition(cPosX,cPosY,heading,
                                        posAhead,posAstar,
                                        SAPI_posPresentationIsRad(),
                                        &posX,&posY);
          if(SAPI_posPresentationIsRad() == 0)
          {                                       /* Presentation in [m] */
           fprintf(stdout,
            "    Beam %03d  Depth: %7.1f PosAhead: %7.1f PosAstar: %7.1f  X: %10.0f m  Y: %10.0f m\n",
                  ii,depth,posAhead,posAstar,posX,posY);
          }
          else               
          {                                       /* Presentation in [rad] */
           fprintf(stdout,
            "    Beam %03d  Depth: %7.1f PosAhead: %7.1f PosAstar: %7.1f  E: %10.7fdeg  N: %10.7fdeg\n",
                  ii,depth,posAhead,posAstar,
                  RAD_TO_PLUMINUS_DEG(posX),
                  RAD_TO_PLUMINUS_DEG(posY) );
          }
          
         }
        }
       }
      } 
     }
    }
    else
    {
     fprintf(stderr,"ExampleERROR: Its not a FAN-Type-Sounder !\n");
    }
   }


   /* SAPI_nextSounding steps to the next sounding                                     */
   /* it reponds with 0 as long as data are readable and as long as EOF is not reached */
   
   status=SAPI_nextSounding(0);
   nrSounding++;
  }





 
  /* SAPI_rewind steps back to the first sounding */
  
  SAPI_rewind(1);
 
  /* Another turn to read sda-Data */

  fprintf(stdout,"\n\n\nANOTHER TURN\n\n\n");
  
  nrSounding = 0;
  status = 0;
  while(status == 0)
  {
   fprintf(stdout,"Sounding Nr %d\n",nrSounding);

   /* looking into Sounding-attached Tables */
   
   if((toTransducer=SAPI_getActualTransducerTable())!=NULL)
   {
    fprintf(stdout,"  Transducerdepth: %5.2f m\n",toTransducer->transducerDepth);
   }
   if((toAngles=SAPI_getActualAngleTable())!=NULL)
   {
    fprintf(stdout,"  Beamangles: %10.7f deg %10.7f deg\n",
         RAD_TO_PLUMINUS_DEG((double)toAngles->beamAngle[0]),
         RAD_TO_PLUMINUS_DEG((double)toAngles->beamAngle[nrBeams-1]));
   }
   if((toCProfile=SAPI_getActualCProfileTable())!=NULL)
   {
    fprintf(stdout,"  CProfile: nrEntries:%d  depth: %7.1fm C-Value: %7.1fm/sec\n",
         toCProfile->numberOfActualValues,
         toCProfile->values[0].depth,
         toCProfile->values[0].cValue);
   }

   /* Another simpler but maybe slower method to get beamdata */

   for(ii=0;ii<10;ii++) /* only few beams , to reduce garbage */
   {
    if(SAPI_getXYZfromMultibeamSounding((long)ii,(long)0,&posY,&posX,&depth) == 0)
    {
     if(SAPI_posPresentationIsRad() == 0)
     {                                       /* Presentation in [m] */
      fprintf(stdout,
       "    Beam %03d  Depth: %7.1f X: %10.0f m  Y: %10.0f m\n",
             ii,depth,posX,posY);
     }
     else               
     {                                       /* Presentation in [rad] */
      fprintf(stdout,
       "    Beam %03d  Depth: %7.1f E: %10.7fdeg  N: %10.7fdeg\n",
             ii,depth,
             RAD_TO_PLUMINUS_DEG(posX),
             RAD_TO_PLUMINUS_DEG(posY) );
     }
    }
   } 
   
   nrSounding++;
   status=SAPI_nextSounding(0);
   if(nrSounding > 3) status = 1;                  /* stop that garbage */
  }





  /* SAPI_close will close files and destroy heap-allocated structures */

  SAPI_close();
 }   
}








/* To run it faster you should run your own version of this Transform,
   precalculating and holding some data for the whole sounding !!!!     */

#define M_PER_RAD_LAT      ((double)(60.0*180.0*1852.0/PI))
#define M_PER_RAD_LON(LAT) (((double)(60.0*180.0*1852.0/PI))*cos(LAT))

#define M_TO_RAD_Y(Y)      ((double)(Y/M_PER_RAD_LAT))
#define M_TO_RAD_X(X,LAT)  ((double)(X/(M_PER_RAD_LON(LAT))))

static void calculateAbsoluteBeamPosition(double posX,double posY,double heading,
                                          double posAhead,double posAstar,
                                          long posPresentation,
                                          double* resultX,double* resultY)
{
 double cosHeading,sinHeading,xM,yM;
 
   cosHeading = cos(heading); 
   sinHeading = sin(heading); 
   xM = (posAhead*sinHeading) + (posAstar * cosHeading);
   yM = (posAhead*cosHeading) - (posAstar * sinHeading); 
   if(posPresentation != 0) /* rad-Presentation */
   {
    yM = M_TO_RAD_Y(yM);
    xM = M_TO_RAD_X(xM,posY);
   }
   *resultX  = posX + xM;
   *resultY  = posY + yM;
}                                          


/* Functions for presenting timedata */


static char* printStartTime(double absTime)
{
 time_t myTime;
 short fractions;
 static char buffer[50]; 

 buffer[0] = '\0';
 myTime = (time_t)absTime;
 fractions = (short)((absTime - (double)(myTime))*1000.0);
 strftime(buffer,40,"%a %b,%d %Y  %H:%M:%S",gmtime(& myTime));
 sprintf(buffer,"%s.%03d",buffer,fractions);
 return(buffer);  
}



static char* printSoundingTime(double absTime)
{
 time_t myTime;
 short fractions;
 static char buffer[50]; 

 buffer[0] = '\0';
 myTime = (time_t)absTime;
 fractions = (short)((absTime - (double)(myTime))*1000.0);
 strftime(buffer,40,"%H:%M:%S",gmtime(& myTime));
 sprintf(buffer,"%s.%03d",buffer,fractions);
 return(buffer);  
}


