/* 
 *           Examplefile for usage of SAPI-Library 
 *
 *      The code is not giving a compact reader, but the target
 *      is to show the usage of many SAPI-functions . 
 *
 *      This example shows a simple reader for vertical data
 *
 *      The executable may be startet through commandlineoptions
 *      or by using one of the attached scripts (*.scr). 
 *
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#ifdef _WIN32
#include <string.h>
#else
#include <string.h>
#include <unistd.h>
#endif
#include <math.h>
#include <time.h>
#include "sapi.h"

/* ERRORPRINT != 0 will force erroroutputs on stderr, if something is going wrong
                   during filehandling                                           */
                   
#define ERRORPRINT 1




/* Some mathematical macros */ 

#ifndef PI
#define PI 3.14159265359
#endif
#define RAD_TO_DEG(a) ((a>=+0.0)?                                \
                                (double)((180.0*a)/PI)           \
                               :                                 \
                                (double)(360.0 - ((-180.0*a)/PI)) )

#define RAD_TO_PLUMINUS_DEG(a) ((double)((180.0*a)/PI))


/* A function for presenting timedata */

static char* printStartTime(double absTime);


char surfDataPath[256] = {" "}; 
char filename[256] = {" "}; 


int main (int argc,char ** argv)
{
 SurfGlobalData*        toGlobalData;

 char* toTypeOfSounder;
 char* sp;
 int n,nrSounding;
 long status;
 double depth,posX,posY;

 /* Get Path and Filename (without extention '.six or .sda') from shell */

 
 for(n=1;n<argc;n++)
 {
  if((sp = strstr(argv[n],"-F")) != NULL) strncpy(filename,sp+2,255);
  if((sp = strstr(argv[n],"-D")) != NULL) strncpy(surfDataPath,sp+2,255);
 }


 /* Maybe you like to printout actual Library-Versions */

 SAPI_printAPIandSURFversion();
  


 /* SAPI_open will open the files and, if successful, give access to six-Data
    and to the first sounding                                                */
    

 if(SAPI_open(surfDataPath,filename,ERRORPRINT) == 0)
 {

  /* Accessing some six-Data */


  /* few basic parameters */
  
  fprintf(stdout,"SURF Name Of Ship   : '%s'\n",SAPI_getNameOfShip());
  fprintf(stdout,"SURF Type Of Sounder: '%s'\n",SAPI_getTypeOfSounder());
  fprintf(stdout,"SURF Name Of Sounder: '%s'\n",SAPI_getNameOfSounder());
  toTypeOfSounder = SAPI_getTypeOfSounder();
  switch(toTypeOfSounder[0])
  {
   case 'M':        /* manual entered data */
   case 'D':        /* digitized data */
   case 'V':        /* vertical sounder */
         break;
   default:
         fprintf(stderr,"\n\n This Profile is not containing vertical data !!!\n");\
         exit(0);
         break;      
  }

  /* How to get more global Informations */

  toGlobalData = SAPI_getGlobalData();
  if(toGlobalData != NULL)
  {
   fprintf(stdout,"SURF-GLOBAL_DATA:\n");
   fprintf(stdout,"    Start Time Of Profile  '%s'\n",
                                          printStartTime(SAPI_getAbsoluteStartTimeOfProfile()));
   fprintf(stdout,"    Ellipsoid  '%s'\n",toGlobalData->nameOfEllipsoid);
   fprintf(stdout,"    Projection '%s'\n",toGlobalData->projection);
  }

  /* Now lets go to read some sda-Data in a simple way (Sounding-Data) */
 
  nrSounding = 0;
  status = 0;
  while(status == 0)    /* This loop is stepping sounding by sounding */
  {

   fprintf(stdout,"Sounding Nr %d\n",nrSounding);

   /* High frequency layer */
   
   if(SAPI_getXYZfromSinglebeamSoundingHF(0,&posY,&posX,&depth) == 0)
   {
     if(SAPI_posPresentationIsRad() == 0)
     {                                       /* Presentation in [m] */
      fprintf(stdout,
       "  HF: Depth: %7.1f X: %10.0f m  Y: %10.0f m\n",depth,posX,posY);
     }
     else               
     {                                       /* Presentation in [rad] */
      fprintf(stdout,
       "  HF: Depth: %7.1f E: %10.7fdeg  N: %10.7fdeg\n",
             depth,
             RAD_TO_PLUMINUS_DEG(posX),
             RAD_TO_PLUMINUS_DEG(posY) );
     }
   }  

   /* Medium frequency layer */
   
   if(SAPI_getXYZfromSinglebeamSoundingMF(0,&posY,&posX,&depth) == 0)
   {
     if(SAPI_posPresentationIsRad() == 0)
     {                                       /* Presentation in [m] */
      fprintf(stdout,
       "  MF: Depth: %7.1f X: %10.0f m  Y: %10.0f m\n",depth,posX,posY);
     }
     else               
     {                                       /* Presentation in [rad] */
      fprintf(stdout,
       "  MF: Depth: %7.1f E: %10.7fdeg  N: %10.7fdeg\n",
             depth,
             RAD_TO_PLUMINUS_DEG(posX),
             RAD_TO_PLUMINUS_DEG(posY) );
     }
   }  

   /* Low frequency layer */
   
   if(SAPI_getXYZfromSinglebeamSoundingLF(0,&posY,&posX,&depth) == 0)
   {
     if(SAPI_posPresentationIsRad() == 0)
     {                                       /* Presentation in [m] */
      fprintf(stdout,
       "  LF: Depth: %7.1f X: %10.0f m  Y: %10.0f m\n",depth,posX,posY);
     }
     else               
     {                                       /* Presentation in [rad] */
      fprintf(stdout,
       "  LF: Depth: %7.1f E: %10.7fdeg  N: %10.7fdeg\n",
             depth,
             RAD_TO_PLUMINUS_DEG(posX),
             RAD_TO_PLUMINUS_DEG(posY) );
     }
   }  
          
   /* SAPI_nextSounding steps to the next sounding                                     */
   /* it reponds with 0 as long as data are readable and as long as EOF is not reached */
   
   status=SAPI_nextSounding(0);
   nrSounding++;
  }



  /* SAPI_close will close files and destroy heap-allocated structures */

  SAPI_close();
 }   
}





/* A function for presenting timedata */


static char* printStartTime(double absTime)
{
 time_t myTime;
 short fractions;
 static char buffer[50]; 

 buffer[0] = '\0';
 myTime = (time_t)absTime;
 fractions = (short)((absTime - (double)(myTime))*1000.0);
 strftime(buffer,40,"%a %b,%d %Y  %H:%M:%S",gmtime(& myTime));
 sprintf(buffer,"%s.%03d",buffer,fractions);
 return(buffer);  
}



