/* 
 *           Examplefile for usage of SAPI-Library 
 *
 *      This example shows how to manipulate and write back
 *      SURF-Data. You should know what you are doing:
 *      YOU ARE MANIPULATING DATA !!
 *      You should also be sure that you are writing the same
 *      SURF-Version as your Offline-system is using.
 *      You may inspect the Version of this Library by
 *           typing :       what libsapi.a
 *      For reading only any SURF-Version equal or higher than
 *      your system's version is o.k. , for writing it must
 *      be the same !!!!
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "sapi.h"

/* ERRORPRINT != 0 will force erroroutputs on stderr, if something is going wrong
                   during filehandling                                           */
                   
#define ERRORPRINT 1




/* Some mathematical macros */ 

#ifndef PI
#define PI 3.14159265359
#endif
#define RAD_TO_DEG(a) ((a>=+0.0)?                                \
                                (double)((180.0*a)/PI)           \
                               :                                 \
                                (double)(360.0 - ((-180.0*a)/PI)) )

#define RAD_TO_PLUMINUS_DEG(a) ((double)((180.0*a)/PI))


#define M_PER_RAD_LAT      ((double)(60.0*180.0*1852.0/PI))
#define M_PER_RAD_LON(LAT) (((double)(60.0*180.0*1852.0/PI))*cos(LAT))

/* simple flat projection */
#define M_TO_RAD_Y(LAT)      ((double)(LAT/M_PER_RAD_LAT))
#define M_TO_RAD_X(LON,LAT)  ((double)(LON/(M_PER_RAD_LON(LAT))))


char surfDataPath[256] = {" "}; 
char filename[256] = {" "}; 


int main (int argc,char ** argv)
{
 SurfGlobalData*        toGlobalData;
 SurfMultiBeamTT*       toTT;
 SurfSingleBeamDepth*   toSBdepth;

 char* sp;
 int n,nrSounding;
 u_short soundingFlag,depthFlag;
 long status,ii,nrBeams;
 double travelTime,depth,refPosX,refPosY;


 /* Get Path and Filename (without extention '.six or .sda') from shell */

 for(n=1;n<argc;n++)
 {
  if((sp = strstr(argv[n],"-F")) != NULL) strncpy(filename,sp+2,255);
  if((sp = strstr(argv[n],"-D")) != NULL) strncpy(surfDataPath,sp+2,255);
 }


 /* Maybe you like to printout actual Library-Versions */

 SAPI_printAPIandSURFversion();
  

 /* SAPI_openIntoMemory will open the files and, if successful, 
    give access to six-Data and to the first sounding.
    ALL DATA are accessible in memory for writing them back to a file  */
    

 if(SAPI_openIntoMemory(surfDataPath,filename,ERRORPRINT) == 0)
 {

  /* Now lets modify some data */

  /* Shift the profile in position */
  
  toGlobalData = SAPI_getGlobalData();
  if(toGlobalData != NULL)
  {
   refPosX=toGlobalData->referenceOfPositionX;
   refPosY=toGlobalData->referenceOfPositionY;

   /* shifting 10km in longitude and 5km in lattitude */

   if(SAPI_posPresentationIsRad() == 0)
   {                                       /* Presentation in [m] */
    toGlobalData->referenceOfPositionX = refPosX + 10000.0;
    toGlobalData->referenceOfPositionY = refPosY + 5000.0;
   }
   else               
   {                                       /* Presentation in [rad] */
    toGlobalData->referenceOfPositionX = refPosX + (M_TO_RAD_X(10000.0,refPosY));
    toGlobalData->referenceOfPositionY = refPosY + (M_TO_RAD_Y(5000.0));
   }
  }


  /* 
     Modifying a depth in Multibeamdata doesn't work, as depth-data
     will be recalculated during 'write back' from angles,traveltimes
     and the other parameters of the fan-geometry .
     So for modifying the depth traveltimes should be changed .
  */
 
  nrSounding = 0;
  nrBeams = SAPI_getNrBeams();
  status = 0;
  while(status == 0)    /* This loop is stepping sounding by sounding */
  {

   if(strcmp(SAPI_getTypeOfSounder(),"F") == 0) /* is it a FAN-Type Sounder ? */
   {
     /* FAN-Type Sounder */
     
     for(ii=0;ii<nrBeams;ii++)
     {
       
      /* Increasing the Traveltimes by 20.0 milliseconds :         */
      /* This creates an invers tunnel-effect on the cross-profile */
        
      if((toTT=SAPI_getMultiBeamTraveltime((long)ii)) != NULL)
      {
       travelTime = (double)(toTT->travelTimeOfRay);
       
       if(travelTime > 0.0)
        toTT->travelTimeOfRay = (float) (travelTime + 0.02);
      }  
     }
   }  


   if(strcmp(SAPI_getTypeOfSounder(),"V") == 0) /* is it a Vertical Sounder ? */
   {
     /* Vertical Sounder */

     /* Increasing the depth by 5m */

     toSBdepth = SAPI_getSingleBeamDepth();
     if(SAPI_dataHaveHighFrequencyLayer())
     {
      depth=(double)toSBdepth->depthHFreq;
      toSBdepth->depthHFreq = (float)(depth+5.0);
     }
     if(SAPI_dataHaveMediumFrequencyLayer())
     {
      depth=(double)toSBdepth->depthMFreq;
      toSBdepth->depthMFreq = (float)(depth+5.0);
     }
     if(SAPI_dataHaveLowFrequencyLayer())
     {
      depth=(double)toSBdepth->depthLFreq;
      toSBdepth->depthLFreq = (float)(depth+5.0);
     }
   }  


   /* SAPI_nextSounding steps to the next sounding                                     */
   /* it reponds with 0 as long as data are readable and as long as EOF is not reached */
   
   status=SAPI_nextSounding(0);
   nrSounding++;
  }

  /* Writing the modified data back to a file: 
     Filename may be the original or just another one */

  SAPI_writeBackFromMemory(surfDataPath,"rewritten",ERRORPRINT);


  /* SAPI_close will close files and destroy heap-allocated structures */

  SAPI_close();
 }   
}



