/* 
 *           Examplefile for usage of SAPI-Library 
 *
 *      This example shows how create own SURF-Data 
 *      for vertical sounders.
 *      You should be sure that you are writing the same
 *      SURF-Version as your Offline-system is using.
 *      You may inspect the Version of this Library by
 *           typing :       what libsapi.a
 *      For 'reading only' any SURF-Version equal or higher than
 *      your system's version is o.k. , for writing it must
 *      be the same !!!!
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "sapi.h"

/* ERRORPRINT != 0 will force erroroutputs on stderr, if something is going wrong
                   during filehandling                                           */
                   
#define ERRORPRINT 1




/* Some mathematical macros */ 

#ifndef PI
#define PI 3.14159265359
#endif

#define DEG_TO_RAD(a)        ((double)((PI*a)/180.0) )

#define M_PER_RAD_LAT      ((double)(60.0*180.0*1852.0/PI))
#define M_PER_RAD_LON(LAT) (((double)(60.0*180.0*1852.0/PI))*cos(LAT))

/* simple flat projection */
#define M_TO_RAD_Y(LAT)      ((double)(LAT/M_PER_RAD_LAT))
#define M_TO_RAD_X(LON,LAT)  ((double)(LON/(M_PER_RAD_LON(LAT))))


char surfDataPath[256] = {""}; 
char filename[256] = {""}; 



int main (int argc,char ** argv)
{
 SurfGlobalData*               toGlobalData;
 SurfTransducerParameterTable* surfTransducerTable;
 SurfPositionAnySensor*        surfPositionSensor;
 SurfSoundingData*             toSounding;
 SurfCenterPosition*           toPositions;
 SurfSingleBeamDepth*          toDepth;
 
 char* sp;
 int n,sounding,nrSoundings;
 u_short soundingFlag,depthFlag;
 long status,ii;
 double depth,posX,posY,relTime;
 u_char ampl;

 posX=posY=0.0;
 relTime=0.0;
 depth = 20.0;
 

 /* Get Path and Filename (without extention '.six or .sda') from shell */

 for(n=1;n<argc;n++)
 {
  if((sp = strstr(argv[n],"-F")) != NULL) strncpy(filename,sp+2,255);
  if((sp = strstr(argv[n],"-D")) != NULL) strncpy(surfDataPath,sp+2,255);
 }

 if(*filename == '\0')sprintf(filename,"newBuildSurfFile");
 if(*surfDataPath == '\0')sprintf(surfDataPath,"../data");

 /* Maybe you like to printout actual Library-Versions */

 SAPI_printAPIandSURFversion();
  

 /* SAPI_createSurfBody creates an empty SURF-file, which may be filled 
    by your data . All records are accessible in memory 
    for writing them back to a file  */

 /* Setting the parameters for your data */

 /* You have got a collection of 279 pings into the water */    

 nrSoundings = 220;

 if(SAPI_createSurfBody(nrSoundings,0,0,ERRORPRINT) == 0)
 {
  /* At first fill in some Global data : */

  if((toGlobalData = SAPI_getGlobalData()) != NULL)
  {
   /* Enter the ship's name (max. 15 characters) */

   sprintf(toGlobalData->shipsName,"Titanic IV");

   /* Enter the start-time of your profile (DDMMYYHHMMSS.NN) */
   /* All times in the profile will be related to this value */

   sprintf(toGlobalData->startTimeOfProfile,"091298123500.00");

   /* Two strings may be used for Dokumentation: (max. 15 characters each) */ 

   sprintf(toGlobalData->regionOfProfile,"my LAKE");
   sprintf(toGlobalData->numberOfProfile,"0001");

   /* You may set the name of your sounder: (max. 15 characters) */ 

   sprintf(toGlobalData->nameOfSounder,"MY_VERTICAL");

   /* You should set the frequencylayer of your data by entering the 
      frequency in Hz to one or more of 3 layers:
                 Low      < 30000Hz
      30000Hz <= Medium  =< 70000Hz
                 High     > 70000Hz 
   */

   toGlobalData->mediumFrequency= 35000.0;
   toGlobalData->highFrequency  =200000.0;

   /* Enter the start-position of your profile in rad WGS84      */
   /* All positions in the profile will be related to this value */

   toGlobalData->referenceOfPositionX=DEG_TO_RAD(125.040); /*longitude 125.040 East */ 
   toGlobalData->referenceOfPositionY=DEG_TO_RAD(-17.350); /*latitude   17.350 South*/ 


   /* Set the Transducer table entries                               */
   /* The Tranducer table describes the position offset [m] of the 
      transducer to that point where center-positions are related to, 
      transducerDepth is the draught of the transducer [m]           
      transducerTwoThetaLM/HFreq is the 2-Theta-angle of the
      transmitbeam                                                   */
      
   surfTransducerTable = SAPI_getActualTransducerTable();

   surfTransducerTable->transducerDepth = 2.40;
   surfTransducerTable->transducerPositionAhead = 5.30;
   surfTransducerTable->transducerPositionStar = -0.30;
   
   surfTransducerTable->transducerTwoThetaLFreq = DEG_TO_RAD(2.0);



   /* Set the Position sensor table entries       */
   /* positionsensor's name  (max. 15 characters) */

   surfPositionSensor = SAPI_getPositionSensor(0l);

   sprintf(surfPositionSensor->positionSensorName,"myGPS");



   /* Fill the sounding data */

   sounding = 0;
   status = 0;
   while(status == 0)    /* This loop is stepping sounding by sounding */
   {
    toSounding = SAPI_getSoundingData();
     
    /* times are relative to toGlobalData->startTimeOfProfile ,
       we are simulating here one ping every second */

    toSounding->relTime = (float)relTime;
    relTime = relTime + 1.0;

    /* tide-values may be attached, if known already in [m] */
                 
    toSounding->tide = (float) 1.29;

    /* the ships heading ( not the course !!) in [rad] */

    toSounding->headingWhileTransmitting = (float) DEG_TO_RAD(45.0);

    /* the heave/roll/pitch-values during transmission [m,rad,rad] */
    /* positive = heave: ship up , pitch: bow up , roll: port up   */

    toSounding->heaveWhileTransmitting = (float) 0.23;
    toSounding->rollWhileTransmitting  = (float) DEG_TO_RAD(0.2);
    toSounding->pitchWhileTransmitting = (float) DEG_TO_RAD(0.1);

    /* cMean is the representative Soundvelocity over the whole way */

    toSounding->cMean = (float) 1493.2;

    /* Positiondata are relative to 'toGlobalData->referenceOfPositionX,Y' */    
    /* we are simulating here a NE-course with about 2m/sec                */

    toPositions=SAPI_getCenterPosition(0);
    toPositions->centerPositionX = (float)posX;
    toPositions->centerPositionY = (float)posY;
    posX = posX + M_TO_RAD_X(2.0,toGlobalData->referenceOfPositionY);
    posY = posY + M_TO_RAD_Y(2.0);
    toPositions->speed = (float) 2.0;

    /* Simulating Depth for medium and high frequency layers */

    toDepth=SAPI_getSingleBeamDepth();
    toDepth->depthHFreq = depth;
    toDepth->depthMFreq = depth + 0.2;

    depth=depth+0.10; /* sim. increasing depth */


    /* SAPI_nextSounding steps to the next sounding                                     */
    /* it reponds with 0 as long as data are readable and as long as EOF is not reached */
   
    status=SAPI_nextSounding(0);
    sounding++;
   }  

   /* Writing the modified data to a file 
      Most missing values will be calculated */

   if(SAPI_writeBackFromMemory(surfDataPath,filename,ERRORPRINT) == 0l)
   {
    fprintf(stderr,"\nExampledata written to:\n    '%s/%s'\n",surfDataPath,filename);
   }    
  }

  /* SAPI_close will close files and destroy heap-allocated structures */

  SAPI_close();
 }   
}



